------------------------------------------------------------------------------
-- --------------------        M a R T E   O S          ------------------- --
------------------------------------------------------------------------------
--                                                             V1.61B   070119
--
--              'K e r n e l . D e v i c e s _ T a b l e . I n i t'
--
--                                   Body
--
--
--  File 'k-devices_table-init.adb'                               By MAR. and
--                                                                   Fguerreira
--
--  This is the last package in the kernel to be elaborated. Here is
--  where interrupts are enabled.
--
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2000-2007, Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael Gonzalez Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------

with Kernel.Devices_Table;
--  pragma Elaborate_All (Kernel.Devices_Table);
--  To force drivers elaboration.

with Kernel.Scheduler;
pragma Elaborate_All (Kernel.Scheduler);
--  Just to force elaboration of the rest of the kernel since
--  'Kernel.Scheduler' is the last package to be elaborated.

with Hardware_Interface;
pragma Elaborate_All (Hardware_Interface);

with Configuration_Parameters;
with Kernel_Console; use Kernel_Console;

package body Kernel.Devices_Table.Init is

   package HWI renames Hardware_Interface;

   use type Int;

   procedure Requires_Body is
   begin
      null;
   end Requires_Body;

begin

   --------------------------
   --  Initialize_Devices  --
   --------------------------
   if Configuration_Parameters.Use_Devices_Filesystem then
      Put ("Devices initialization..."); New_Line;
      for I in Major loop
         if The_Driver_Table (I).Create /= null then
            Put ("   Major Number "); Put (Integer (I));
            case I is
               when 1 =>      Put (" (stdin)  ");
               when 2 =>      Put (" (stdout) ");
               when 3 =>      Put (" (stderr) ");
               when others => Put ("          ");
            end case;
            Put (The_Driver_Table (I).Name);
            if The_Driver_Table (I).Create.all = 0 then
               Put ("...OK");
            else
               Put ("...*** FAIL!! ***");
            end if;
            New_Line;
         end if;
      end loop;
   end if;

   --  Call 'End_Of_Kernel_Initialization'
   Kernel_Console.End_Of_Kernel_Initialization;

   --  Enable interrupts
   HWI.Interrupt_Controller_Hardware_Enabled;
   HWI.Interrupts_Enabled;

end Kernel.Devices_Table.Init;
