------------------------------------------------------------------------------
--  ------------------         M a R T E   O S         -------------------  --
------------------------------------------------------------------------------
--                                                             V1.61B   070119
--
--      'K e r n e l . T i m e d _ H a n d l e r s .
--                                       A d a _ T i m i n g _ E v e n t s'
--
--                                   Spec
--
--
--
--  File 'kernel-timed_handlers-ada_timing_events.ads'                 By MAR.
--
--  Support for Ada 2005 standard packages 'Ada.Execution_Time.Timers' and
--  'Ada.Timing_Events'.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2000-2007, Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael Gonzalez Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------
with Kernel.Timed_Handlers.Operations;

with Hardware_Interface;
with Kernel.Timed_Handlers;
with Ada.Finalization;

package Kernel.Timed_Handlers.Ada_Timing_Events is

   package HWI renames Hardware_Interface;
   package TH  renames Kernel.Timed_Handlers;

   type Base_Timing_Event;

   type Base_Timing_Event_Handler
     is access protected procedure (Event : in out Base_Timing_Event);

   type Base_Timing_Event is
     new Ada.Finalization.Limited_Controlled with record
      Cleared : Boolean;
      Handler : Base_Timing_Event_Handler;
      Timed_Handler : aliased Kernel.Timed_Handlers.Timed_Handler_Timed_Event;
     end record;

   procedure Set_Handler_HWTime (Event   : in out Base_Timing_Event;
                                 T       : in Hardware_Interface.HWTime;
                                 Handler : in Base_Timing_Event_Handler;
                                 Options : in TH.Timed_Handler_Options);
   pragma Inline (Set_Handler_HWTime);

   function Current_Handler (Event : Base_Timing_Event)
                                 return Base_Timing_Event_Handler;

   procedure Cancel_Handler (Event     : in out Base_Timing_Event;
                             Cancelled : out Boolean);

   procedure Initialize (Event : in out Base_Timing_Event);
   procedure Finalize   (Event : in out Base_Timing_Event);

end Kernel.Timed_Handlers.Ada_Timing_Events;
