------------------------------------------------------------------------------
--                                                                          --
--                GNU ADA RUN-TIME LIBRARY (GNARL) COMPONENTS               --
--                                                                          --
--                   S Y S T E M . O S _ I N T E R F A C E                  --
--                                                                          --
--                                  S p e c                                 --
--                                                                          --
--                             $Revision: 53 $
--                                                                          --
--          Copyright (C) 1991-2000 Free Software Foundation, Inc.          --
--                                                                          --
-- GNARL is free software; you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. GNARL is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with GNARL; see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
-- GNARL was developed by the GNARL team at Florida State University. It is --
-- now maintained by Ada Core Technologies Inc. in cooperation with Florida --
-- State University (http://www.gnat.com).                                  --
--                                                                          --
------------------------------------------------------------------------------
------------------------------------------------------------------------------
--  ------------------         M a R T E   O S         -------------------  --
------------------------------------------------------------------------------
--                                                             V1.61B   070119
--
--  This is a MaRTE OS version of this package.
--
--  (Thanks to Javier Miranda (Universidad de Las Palmas de Gran Canaria)
--  for its help in this adaptation).
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2000-2007, Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael Gonzalez Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------

--  This package encapsulates all direct interfaces to OS services
--  that are needed by children of System.

--  PLEASE DO NOT add any with-clauses to this package
--  or remove the pragma Elaborate_Body.
--  It is designed to be a bottom-level (leaf) package.

with Interfaces.C;

--  << MaRTE OS
with Pthread;
with Kernel.Tasks_Operations.Attributes;
with Kernel.Mutexes;
with Kernel.Tasks_Operations.Nanosleep_Gnat;
with Time;
with Signal;
with Sched;
with General_Constants;
with Error_Codes;
with Signals_Values;
--  MaRTE OS >>

package System.OS_Interface is

   --  << MaRTE OS
   --  pragma Preelaborate;
   pragma Elaborate_Body;
   --  MaRTE OS >>

   --  << MaRTE OS
   --  pragma Linker_Options ("-lgthreads");
   --  pragma Linker_Options ("-lmalloc");
   --  MaRTE OS >>

   --  << MaRTE OS
   package PSignal renames Signal;
   --  To avoid confusion with type 'Signal'
   --  MaRTE OS >>

   subtype int            is Interfaces.C.int;
   subtype short          is Interfaces.C.short;
   subtype long           is Interfaces.C.long;
   subtype unsigned       is Interfaces.C.unsigned;
   subtype unsigned_short is Interfaces.C.unsigned_short;
   subtype unsigned_long  is Interfaces.C.unsigned_long;
   subtype unsigned_char  is Interfaces.C.unsigned_char;
   subtype plain_char     is Interfaces.C.plain_char;
   subtype size_t         is Interfaces.C.size_t;

   -----------
   -- Errno --
   -----------

   function errno return int;
   --  << MaRTE OS
   --  pragma Import (C, errno, "__get_errno");
   --  MaRTE OS >>

   --  NAMEs not used are commented-out
   --  NAMEs not supported on this system have __NAME for value

   --  E2BIG    : constant := 7;
   --  EACCES   : constant := 13;

   --  << MaRTE OS
   --  EAGAIN   : constant := 11;
   EAGAIN   : constant := Error_Codes.RESOURCE_TEMPORARILY_UNAVAILABLE;
   --  MaRTE OS >>

   --  EBADF    : constant := 9;
   --  EBUSY    : constant := 16;
   --  ECHILD   : constant := 10;
   --  EDEADLK  : constant := 45;
   --  EDOM     : constant := 33;
   --  EEXIST   : constant := 17;
   --  EFAULT   : constant := 14;
   --  EFBIG    : constant := 27;

   --  << MaRTE OS
   --  EINTR    : constant := 4;
   --  EINVAL   : constant := 22;
   EINTR    : constant := Error_Codes.INTERRUPTED_OPERATION;
   EINVAL   : constant := Error_Codes.INVALID_ARGUMENT;
   --  MaRTE OS >>

   --  EIO      : constant := 5;
   --  EISDIR   : constant := 21;
   --  EMFILE   : constant := 24;
   --  EMLINK   : constant := 31;
   --  ENAMETOOLONG : constant := 78;
   --  ENFILE   : constant := 23;
   --  ENODEV   : constant := 19;
   --  ENOENT   : constant := 2;
   --  ENOEXEC  : constant := 8;
   --  ENOLCK   : constant := 46;

   --  << MaRTE OS
   --  ENOMEM   : constant := 12;
   ENOMEM   : constant := Error_Codes.NOT_ENOUGH_SPACE;
   --  MaRTE OS >>

   --  ENOSPC   : constant := 28;
   --  ENOSYS   : constant := 89;
   --  ENOTDIR  : constant := 20;
   --  ENOTEMPTY    : constant := 93;
   --  ENOTTY   : constant := 25;
   --  ENXIO    : constant := 6;
   --  EPERM    : constant := 1;
   --  EPIPE    : constant := 32;
   --  ERANGE   : constant := 34;
   --  EROFS    : constant := 30;
   --  ESPIPE   : constant := 29;
   --  ESRCH    : constant := 3;

   --  << MaRTE OS
   --  ETIMEDOUT    : constant := 110;
   ETIMEDOUT    : constant := Error_Codes.TIMED_OUT;
   --  MaRTE OS >>

   --  EXDEV    : constant := 18;

   -------------
   -- Signals --
   -------------

   --  << MaRTE OS -----------------------------------------------------------
   --  NSIG : constant := 32;
   --  type Signal is new int range 0 .. Interfaces.C."-" (NSIG, 1);
   --  for Signal'Size use int'Size;
   --  SIGHUP  : constant := 1; --  hangup
   --  SIGINT  : constant := 2; --  interrupt (rubout)
   --  SIGQUIT : constant := 3; --  quit (ASCD FS)
   --  SIGILL  : constant := 4; --  illegal instruction (not reset)
   --  SIGTRAP : constant := 5; --  trace trap (not reset)
   --  SIGIOT  : constant := 6; --  IOT instruction
   --  SIGABRT : constant := 6; -- used by abort, replace SIGIOT in the  future
   --  SIGFPE  : constant := 8; --  floating point exception
   --  SIGKILL : constant := 9; --  kill (cannot be caught or ignored)
   --  SIGBUS  : constant := 7; --  bus error
   --  SIGSEGV : constant := 11; --  segmentation violation
   --  SIGPIPE : constant := 13; --  write on a pipe with no one to read it
   --  SIGALRM : constant := 14; --  alarm clock
   --  SIGTERM : constant := 15; --  software termination signal from kill
   --  SIGUSR1 : constant := 10; --  user defined signal 1
   --  SIGUSR2 : constant := 12; --  user defined signal 2
   --  SIGCLD  : constant := 17; --  alias for SIGCHLD
   --  SIGCHLD : constant := 17; --  child status change
   --  SIGPWR  : constant := 30; --  power-fail restart
   --  SIGWINCH: constant := 28; --  window size change
   --  SIGURG  : constant := 23; --  urgent condition on IO channel
   --  SIGPOLL : constant := 29; --  pollable event occurred
   --  SIGIO   : constant := 29; --  I/O now possible (4.2 BSD)
   --  SIGLOST : constant := 29; --  File lock lost
   --  SIGSTOP : constant := 19; --  stop (cannot be caught or ignored)
   --  SIGTSTP : constant := 20; --  user stop requested from tty
   --  SIGCONT : constant := 18; --  stopped process has been continued
   --  SIGTTIN : constant := 21; --  background tty read attempted
   --  SIGTTOU : constant := 22; --  background tty write attempted
   --  SIGVTALRM : constant := 26; --  virtual timer expired
   --  SIGPROF   : constant := 27; --  profiling timer expired
   --  SIGXCPU   : constant := 24; --  CPU time limit exceeded
   --  SIGXFSZ   : constant := 25; --  filesize limit exceeded
   --  SIGUNUSED : constant := 31; --  unused signal (Linux)
   --  SIGSTKFLT : constant := 16; --  coprocessor stack fault (Linux)

   NSIG : constant := PSignal.Signal'Last;
   subtype Signal is PSignal.Signal;

   SIGHUP     : constant :=
     Signals_Values.SIGHUP; --  hangup
   SIGINT     : constant := -- 2
     Signals_Values.SIGINT; --  interrupt (rubout)
   SIGQUIT    : constant := -- 3
     Signals_Values.SIGQUIT; --  quit (ASCD FS)
   SIGILL     : constant := -- 4
     Signals_Values.SIGILL; --  illegal instruction (not reset)
   SIGTRAP    : constant := -- 5
     Signals_Values.SIGTRAP; --  trace trap (not reset)
   --  SIGIOT     : constant := -- 6
   --    Signals_Values.SIGIOT; --  IOT instruction
   SIGABRT    : constant := -- 6
     Signals_Values.SIGABRT; --  used by abort, replace SIGIOT in the  future
   SIGFPE     : constant := -- 8
     Signals_Values.SIGFPE; --  floating point exception
   SIGKILL    : constant := -- 9
     Signals_Values.SIGKILL; --  kill (cannot be caught or ignored)
   SIGBUS     : constant := -- 7
     Signals_Values.SIGBUS; --  bus error
   SIGSEGV    : constant := -- 11
     Signals_Values.SIGSEGV; --  segmentation violation
   SIGPIPE    : constant := -- 13
     Signals_Values.SIGPIPE; --  write on a pipe with no one to read it
   SIGALRM    : constant := -- 14
     Signals_Values.SIGALRM; --  alarm clock
   SIGTERM    : constant := -- 15
     Signals_Values.SIGTERM; --  software termination signal from kill
   SIGUSR1    : constant := -- 10
     Signals_Values.SIGUSR1; --  user defined signal 1
   SIGUSR2    : constant := -- 12
     Signals_Values.SIGUSR2; --  user defined signal 2
   --  SIGCLD     : constant := -- 17
   --    Signals_Values.SIGCLD; --  alias for SIGCHLD
   SIGCHLD    : constant := -- 17
     Signals_Values.SIGCHLD; --  child status change
   --  SIGPWR     : constant := -- 30
   --    Signals_Values.SIGPWR; --  power-fail restart
   --  SIGWINCH   : constant := -- 28
   --    Signals_Values.SIGWINCH; --  window size change
   --  SIGURG     : constant := -- 23
   --    Signals_Values.SIGURG; --  urgent condition on IO channel
   --  SIGPOLL    : constant := -- 29
   --    Signals_Values.SIGPOLL; --  pollable event occurred
   --  SIGIO      : constant := -- 29
   --    Signals_Values.SIGIO; --  I/O now possible (4.2 BSD)
   --  SIGLOST    : constant := -- 29
   --    Signals_Values.SIGLOST; --  File lock lost
   SIGSTOP    : constant := -- 19
     Signals_Values.SIGSTOP; --  stop (cannot be caught or ignored)
   SIGTSTP    : constant := -- 20
     Signals_Values.SIGTSTP; --  user stop requested from tty
   SIGCONT    : constant := -- 18
     Signals_Values.SIGCONT; --  stopped process has been continued
   SIGTTIN    : constant := -- 21
     Signals_Values.SIGTTIN; --  background tty read attempted
   SIGTTOU    : constant := -- 22
     Signals_Values.SIGTTOU; --  background tty write attempted
   SIGVTALRM  : constant := -- 26
     Signals_Values.SIGVTALRM; --  virtual timer expired
   --  SIGPROF    : constant := -- 27
   --    Signals_Values.SIGPROF; --  profiling timer expired
   --  SIGXCPU    : constant := -- 24
   --    Signals_Values.SIGXCPU; --  CPU time limit exceeded
   --  SIGXFSZ    : constant := -- 25
   --    Signals_Values.SIGXFSZ; --  filesize limit exceeded
   SIGUNUSED  : constant := -- 31
     Signals_Values.SIGUNUSED;  --  unused signal (Linux)
   --  SIGSTKFLT  : constant := -- 16
   --    Signals_Values.SIGSTKFLT;  --  coprocessor stack fault (Linux)
   --  MaRTE OS >>
   --  -----------------------------------------------------------------------




   SIGADAABORT : constant := SIGABRT;
   --  Change this if you want to use another signal for task abort.
   --  SIGTERM might be a good one.

   type Signal_Set is array (Natural range <>) of Signal;

   Unmasked    : constant Signal_Set :=
   --  << MaRTE OS
   --  (SIGTRAP, SIGBUS, SIGTTIN, SIGTTOU, SIGTSTP, SIGPROF);
     (SIGTRAP, SIGBUS);
   --  MaRTE OS >>

   Reserved    : constant Signal_Set :=
     (SIGKILL, SIGSTOP, SIGALRM, SIGVTALRM, SIGUNUSED);

   type sigset_t is private;

   function sigaddset
     (set : access sigset_t;
      sig : Signal)
     return int;
   --  << MaRTE OS
   --  pragma Import (C, sigaddset, "sigaddset");
   --  MaRTE OS >>

   function sigdelset
     (set : access sigset_t;
      sig : Signal)
     return int;
   --  << MaRTE OS
   --  pragma Import (C, sigdelset, "sigdelset");
   --  MaRTE OS >>

   function sigfillset
     (set : access sigset_t)
     return int;
   --  << MaRTE OS
   --  pragma Import (C, sigfillset, "sigfillset");
   --  MaRTE OS >>

   function sigismember
     (set : access sigset_t;
      sig : Signal)
     return int;
   --  << MaRTE OS
   --  pragma Import (C, sigismember, "sigismember");
   --  MaRTE OS >>

   function sigemptyset
     (set : access sigset_t)
     return int;
   --  << MaRTE OS
   --  pragma Import (C, sigemptyset, "sigemptyset");
   --  MaRTE OS >>

   type union_type_3 is new String (1 .. 116);
   type siginfo_t is record
      si_signo     : int;
      si_code      : int;
      si_errno     : int;
      X_data       : union_type_3;
   end record;
   pragma Convention (C, siginfo_t);

   type struct_sigaction is record
      sa_handler   : System.Address;
      sa_mask      : sigset_t;
      sa_flags     : unsigned_long;
      sa_restorer  : System.Address;
   end record;
   --  << MaRTE OS
   --  pragma Convention (C, struct_sigaction);
   --  MaRTE OS >>
   type struct_sigaction_ptr is access all struct_sigaction;

   type Machine_State is record
      eip : unsigned_long;
      ebx : unsigned_long;
      esp : unsigned_long;
      ebp : unsigned_long;
      esi : unsigned_long;
      edi : unsigned_long;
   end record;
   type Machine_State_Ptr is access all Machine_State;

   --  << MaRTE OS
   --  SIG_BLOCK   : constant := 0;
   --  SIG_UNBLOCK : constant := 1;
   --  SIG_SETMASK : constant := 2;
   SIG_BLOCK   : constant int := int (PSignal.SIG_BLOCK);
   SIG_UNBLOCK : constant int := int (PSignal.SIG_UNBLOCK);
   SIG_SETMASK : constant int := int (PSignal.SIG_SETMASK);
   --  MaRTE OS >>

   --  SA_NOCLDSTOP : constant := 1;
   --  not used
   --  SA_SIGINFO : constant := __SA_SIGINFO;
   --  not used

   --  << MaRTE OS
   --  SIG_DFL : constant := 0;
   --  SIG_IGN : constant := 1;
   SIG_DFL : constant int := int (PSignal.SIG_DFL);
   SIG_IGN : constant int := int (PSignal.SIG_IGN);
   --  MaRTE OS >>

   --  SIG_ERR : constant := -1;
   --  not used

   function sigaction
     (sig  : Signal;
      act  : struct_sigaction_ptr;
      oact : struct_sigaction_ptr)
     return int;
   --  << MaRTE OS
   --  pragma Import (C, sigaction, "sigaction");
   --  MaRTE OS >>

   ----------
   -- Time --
   ----------

   Time_Slice_Supported : constant Boolean := False;
   --  Indicates wether time slicing is supported (i.e FSU threads have been
   --  compiled with DEF_RR)

   --  << MaRTE OS
   --  type timespec is private;
   subtype timespec is Duration;
   --  MaRTE OS>>

   function nanosleep (rqtp, rmtp : access timespec) return int
     renames Kernel.Tasks_Operations.Nanosleep_Gnat.Nanosleep_Gnat;
   --  << MaRTE OS
   --  pragma Inline (nanosleep);
   --  MaRTE OS >>

   subtype clockid_t is Time.Clockid_T;
   CLOCK_REALTIME : constant clockid_t := Time.CLOCK_REALTIME;
   --  << MaRTE OS
   --  type clockid_t is private;
   --  CLOCK_REALTIME : constant clockid_t;
   --  MaRTE OS >>

   function clock_gettime
     (clock_id : clockid_t;
      tp       : access timespec)
     return int;
   --  << MaRTE OS
   --  pragma Import (C, clock_gettime, "clock_gettime");
   --  MaRTE OS >>

   function To_Duration (TS : timespec) return Duration;
   pragma Inline (To_Duration);

   function To_Timespec (D : Duration) return timespec;
   pragma Inline (To_Timespec);

   --  << MaRTE OS
   --  type struct_timezone is record
   --     tz_minuteswest  : int;
   --     tz_dsttime   : int;
   --  end record;
   --  pragma Convention (C, struct_timezone);
   --  type struct_timeval is private;
   subtype struct_timezone is Time.Timezone;
   subtype struct_timeval is Time.Timeval;
   --  This is needed on systems that do not have clock_gettime()
   --  but do have gettimeofday().
   --  MaRTE OS >>

   --  << MaRTE OS
   --  function To_Duration (TV : struct_timeval) return Duration;
   --  pragma Inline (To_Duration);
   --  function To_Timeval (D : Duration) return struct_timeval;
   --  pragma Inline (To_Timeval);
   function To_Duration (TV : struct_timeval) return Duration
     renames Time.To_Duration;
   pragma Inline (To_Duration);

   function To_Timeval (D : Duration) return struct_timeval
     renames Time.To_Timeval;
   pragma Inline (To_Timeval);
   --  MaRTE OS >>


   function gettimeofday
     (tv : access struct_timeval;
      tz : access struct_timezone) return int
     renames Time.Gettimeofday;
   --  << MaRTE OS
   --  pragma Import (C, gettimeofday, "gettimeofday");
   --  MaRTE OS >>

   --  add a hook to locate the Epoch, for use with Calendar????

   -------------------------
   -- Priority Scheduling --
   -------------------------

   MIN_PRIO : constant := 0;
   MAX_PRIO : constant := 0;

   --  << MaRTE OS
   --  SCHED_FIFO  : constant := 0;
   --  SCHED_RR    : constant := 1;
   --  SCHED_OTHER : constant := 2;
   SCHED_FIFO  : constant := General_Constants.SCHED_FIFO;
   SCHED_RR    : constant := General_Constants.SCHED_RR;
   SCHED_OTHER : constant := General_Constants.SCHED_OTHER;
   --  MaRTE OS >>


   -------------
   -- Process --
   -------------

   --  << MaRTE OS
   --  type pid_t is private;
   subtype pid_t is int;
   --  MaRTE OS >>

   function kill
     (pid : pid_t;
      sig : Signal)
   return int
     renames PSignal.Kill;
   --  << MaRTE OS
   --  pragma Import (C, kill, "kill");
   --  MaRTE OS >>

   function getpid return pid_t;
   --  << MaRTE OS
   --  pragma Import (C, getpid, "getpid");
   --  MaRTE OS >>


   ---------
   -- LWP --
   ---------
   --  << MaRTE OS
   --  This is a new service of the GNAT interface (J.Miranda)
   --  MaRTE OS>>

   function lwp_self return System.Address;
   --  lwp_self does not exist on this thread library, revert to pthread_self
   --  which is the closest approximation (with getpid). This function is
   --  needed to share 7staprop.adb across POSIX-like targets.
   --  << MaRTE OS
   --  pragma Import (C, lwp_self, "pthread_self");
   --  MaRTE OS >>

   -------------
   -- Threads --
   -------------

   type Thread_Body is access
     function (arg : System.Address) return System.Address;
   type pthread_t           is private;
   subtype Thread_Id        is pthread_t;

   type pthread_mutex_t     is limited private;

   --  << MaRTE OS
   --  type pthread_cond_t      is limited private;
   --  type pthread_attr_t      is limited private;
   --  type pthread_mutexattr_t is limited private;
   --  type pthread_condattr_t  is limited private;
   --  type pthread_key_t       is private;
   subtype pthread_cond_t      is Pthread.Pthread_Cond_T;
   subtype pthread_attr_t      is Pthread.Pthread_Attr_T;
   subtype pthread_mutexattr_t is Pthread.Pthread_Mutexattr_T;
   subtype pthread_condattr_t  is Pthread.Pthread_Condattr_T;
   subtype pthread_key_t       is Pthread.Pthread_Key_T;
   --  MaRTE OS >>

   Self_Key : pthread_key_t;


   --  << MaRTE OS
   --  PTHREAD_CREATE_DETACHED : constant := 1;
   --  PTHREAD_CREATE_JOINABLE : constant := 0;
   PTHREAD_CREATE_DETACHED : constant :=
     General_Constants.PTHREAD_CREATE_DETACHED;
   PTHREAD_CREATE_JOINABLE : constant :=
     General_Constants.PTHREAD_CREATE_JOINABLE;
   --  MaRTE OS >>

   -----------
   -- Stack --
   -----------

   Stack_Base_Available : constant Boolean := False;
   --  Indicates wether the stack base is available on this target.
   --  This allows us to share s-osinte.adb between all the FSU run time.
   --  Note that this value can only be true if pthread_t has a complete
   --  definition that corresponds exactly to the C header files.

   function Get_Stack_Base (thread : pthread_t) return Address;
   pragma Inline (Get_Stack_Base);
   --  returns the stack base of the specified thread.
   --  Only call this function when Stack_Base_Available is True.

   function Get_Page_Size return size_t;
   function Get_Page_Size return Address;
   --  << MaRTE OS
   --  pragma Import (C, Get_Page_Size, "getpagesize");
   --  MaRTE OS >>
   --  returns the size of a page, or 0 if this is not relevant on this
   --  target

   PROT_NONE  : constant := 0;
   PROT_READ  : constant := 1;
   PROT_WRITE : constant := 2;
   PROT_EXEC  : constant := 4;
   PROT_ALL   : constant := PROT_READ + PROT_WRITE + PROT_EXEC;

   PROT_ON    : constant := PROT_NONE;
   PROT_OFF   : constant := PROT_ALL;

   function mprotect
     (addr : Address; len : size_t; prot : int) return int;
   --  << MaRTE OS
   --  pragma Import (C, mprotect);
   --  MaRTE OS >>

   -----------------------------------------
   --  Nonstandard Thread Initialization  --
   -----------------------------------------
   --  FSU_THREADS requires pthread_init, which is nonstandard
   --  and this should be invoked during the elaboration of s-taprop.adb
   procedure pthread_init;
   --  << MaRTE OS
   --  pragma Import (C, pthread_init, "pthread_init");
   --  MaRTE OS >>

   ---------------------------
   --  POSIX.1c  Section 3  --
   ---------------------------

   function sigwait
     (set : access sigset_t;
      sig : access Signal)
     return int;
   --  << MaRTE OS
   --  pragma Inline (sigwait);
   --  MaRTE OS >>
   --  FSU_THREADS has a nonstandard sigwait

   function pthread_kill
     (thread : pthread_t;
      sig    : Signal)
     return   int;
   --  << MaRTE OS
   --  pragma Import (C, pthread_kill, "pthread_kill");
   --  MaRTE OS >>

   --  FSU threads does not have pthread_sigmask. Instead, it uses
   --  sigprocmask to do the signal handling when the thread library is
   --  sucked in.

   type sigset_t_ptr is access all sigset_t;

   function pthread_sigmask
     (how  : int;
      set  : sigset_t_ptr;
      oset : sigset_t_ptr)
      return int;
   --  << MaRTE OS
   --  pragma Import (C, pthread_sigmask, "sigprocmask");
   --  MaRTE OS >>

   ----------------------------
   --  POSIX.1c  Section 11  --
   ----------------------------

   function pthread_mutexattr_init
     (attr : access pthread_mutexattr_t)
     return int
     renames Pthread.Pthread_Mutexattr_Init;
   --  << MaRTE OS
   --  pragma Import (C, pthread_mutexattr_init, "pthread_mutexattr_init");
   --  MaRTE OS >>

   function pthread_mutexattr_destroy
     (attr : access pthread_mutexattr_t)
     return int
     renames Pthread.Pthread_Mutexattr_Destroy;
   --  << MaRTE OS
   --  pragma Import (C, pthread_mutexattr_destroy,
   --                    "pthread_mutexattr_destroy");
   --  MaRTE OS >>

   function pthread_mutex_init
     (mutex : access pthread_mutex_t;
      attr  : access pthread_mutexattr_t)
     return int;
   --  << MaRTE OS
   --  pragma Import (C, pthread_mutex_init, "pthread_mutex_init");
   --  MaRTE OS >>

   function pthread_mutex_destroy
     (mutex : access pthread_mutex_t)
     return  int;
   --  << MaRTE OS
   --  pragma Import (C, pthread_mutex_destroy, "pthread_mutex_destroy");
   --  MaRTE OS >>

   function pthread_mutex_lock
     (mutex : access pthread_mutex_t)
     return int;
   --  << MaRTE OS
   --  pragma Inline (pthread_mutex_lock);
   --  MaRTE OS >>
   --  FSU_THREADS has nonstandard pthread_mutex_lock

   function pthread_mutex_unlock
     (mutex : access pthread_mutex_t)
     return int;
   --  << MaRTE OS
   --  pragma Inline (pthread_mutex_unlock);
   --  MaRTE OS >>
   --  FSU_THREADS has nonstandard pthread_mutex_lock

   function pthread_condattr_init
     (attr : access pthread_condattr_t)
     return int
     renames Pthread.Pthread_Condattr_Init;
   --  << MaRTE OS
   --  pragma Import (C, pthread_condattr_init, "pthread_condattr_init");
   --  MaRTE OS >>

   function pthread_condattr_destroy
     (attr : access pthread_condattr_t)
     return int
     renames Pthread.Pthread_Condattr_Destroy;
   --  << MaRTE OS
   --  pragma Import (C, pthread_condattr_destroy, "pthread_condattr_destroy");
   --  MaRTE OS >>

   function pthread_cond_init
     (cond : access pthread_cond_t;
      attr : access pthread_condattr_t)
     return int
     renames Pthread.Pthread_Cond_Init;
   --  << MaRTE OS
   --  pragma Import (C, pthread_cond_init, "pthread_cond_init");
   --  MaRTE OS >>

   function pthread_cond_destroy
     (cond : access pthread_cond_t)
     return int
     renames Pthread.Pthread_Cond_Destroy;
   --  << MaRTE OS
   --  pragma Import (C, pthread_cond_destroy, "pthread_cond_destroy");
   --  MaRTE OS >>

   function pthread_cond_signal
     (cond : access pthread_cond_t)
     return int
     renames Pthread.Pthread_Cond_Signal;
   function pthread_cond_wait
     (cond  : access pthread_cond_t;
      mutex : access pthread_mutex_t)
     return  int;
   --  << MaRTE OS
   --  pragma Inline (pthread_cond_wait);
   --  MaRTE OS >>
   --  FSU_THREADS has a nonstandard pthread_cond_wait

   function pthread_cond_timedwait
     (cond    : access pthread_cond_t;
      mutex   : access pthread_mutex_t;
      abstime : access timespec)
     return    int;
   --  << MaRTE OS
   --  pragma Inline (pthread_cond_timedwait);
   --  MaRTE OS >>
   --  FSU_THREADS has a nonstandard pthread_cond_timedwait

   Relative_Timed_Wait : constant Boolean := False;
   --  pthread_cond_timedwait requires an absolute delay time

   ----------------------------
   --  POSIX.1c  Section 13  --
   ----------------------------

   --  << MaRTE OS
   --  PTHREAD_PRIO_NONE    : constant := 0;
   --  PTHREAD_PRIO_PROTECT : constant := 2;
   --  PTHREAD_PRIO_INHERIT : constant := 1;
   PTHREAD_PRIO_NONE    : constant :=
     General_Constants.NO_PRIORITY_INHERITANCE;
   PTHREAD_PRIO_PROTECT : constant :=
     General_Constants.HIGHEST_CEILING_PRIORITY;
   PTHREAD_PRIO_INHERIT : constant :=
     General_Constants.HIGHEST_BLOCKED_TASK;
   --  MaRTE OS >>

   function pthread_mutexattr_setprotocol
     (attr     : access pthread_mutexattr_t;
      protocol : int)
     return int
     renames Pthread.Pthread_Mutexattr_Setprotocol;
   --  << MaRTE OS
   --  pragma Import
   --     (C, pthread_mutexattr_setprotocol, "pthread_mutexattr_setprotocol");
   --  MaRTE OS >>

   function pthread_mutexattr_getprotocol
     (attr     : access pthread_mutexattr_t;
      protocol : access int)
     return int
     renames Kernel.Mutexes.Pthread_Mutexattr_Getprotocol;
   --  << MaRTE OS
   --  pragma Import
   --    (C, pthread_mutexattr_getprotocol, "pthread_mutexattr_getprotocol");
   --  MaRTE OS >>

   function pthread_mutexattr_setprioceiling
     (attr     : access pthread_mutexattr_t;
      prioceiling : int)
     return int
     renames Pthread.Pthread_Mutexattr_Setprioceiling;
   --  << MaRTE OS
   --  pragma Import
   --    (C, pthread_mutexattr_setprioceiling,
   --       "pthread_mutexattr_setprio_ceiling");
   --  MaRTE OS >>

   function pthread_mutexattr_getprioceiling
     (attr     : access pthread_mutexattr_t;
      prioceiling : access int)
     return int
     renames Kernel.Mutexes.Pthread_Mutexattr_Getprioceiling;
   --  << MaRTE OS
   --  pragma Import
   --    (C, pthread_mutexattr_getprioceiling,
   --       "pthread_mutexattr_getprio_ceiling");
   --  MaRTE OS >>

   type struct_sched_param is record
      sched_priority : int;  --  scheduling priority
   end record;

   function pthread_getschedparam
     (thread : pthread_t;
      policy : access int;
      param  : access struct_sched_param)
     return int;
   --  << MaRTE OS
   --  pragma Inline (pthread_getschedparam);
   --  MaRTE OS >>
   --  FSU_THREADS does not have pthread_getschedparam

   function pthread_setschedparam
     (thread : pthread_t;
      policy : int;
      param  : access struct_sched_param)
     return int;
   --  << MaRTE OS
   --  pragma Inline (pthread_setschedparam);
   --  MaRTE OS >>
   --  FSU_THREADS does not have pthread_setschedparam

   function pthread_attr_setscope
     (attr            : access pthread_attr_t;
      contentionscope : int)
     return int
     renames Pthread.Pthread_Attr_Setscope;
   --  << MaRTE OS
   --  pragma Import (C, pthread_attr_setscope, "pthread_attr_setscope");
   --  MaRTE OS >>

   function pthread_attr_getscope
     (attr            : access pthread_attr_t;
      contentionscope : access int)
     return int
     renames Pthread.Pthread_Attr_Getscope;
   --  << MaRTE OS
   --  pragma Import (C, pthread_attr_getscope, "pthread_attr_getscope");
   --  MaRTE OS >>

   function pthread_attr_setinheritsched
     (attr         : access pthread_attr_t;
      inheritsched : int)
     return int
     renames Kernel.Tasks_Operations.Attributes.Pthread_Attr_Setinheritsched;
   --  << MaRTE OS
   --  pragma Import
   --    (C, pthread_attr_setinheritsched, "pthread_attr_setinheritsched");
   --  MaRTE OS >>

   function pthread_attr_getinheritsched
     (attr         : access pthread_attr_t;
      inheritsched : access int)
     return int
     renames Kernel.Tasks_Operations.Attributes.Pthread_Attr_Getinheritsched;
   --  << MaRTE OS
   --  pragma Import
   --    (C, pthread_attr_getinheritsched, "pthread_attr_getinheritsched");
   --  MaRTE OS >>

   function pthread_attr_setschedpolicy
     (attr   : access pthread_attr_t;
      policy : int)
     return int
     renames Kernel.Tasks_Operations.Attributes.Pthread_Attr_Setschedpolicy;
   --  << MaRTE OS
   --  pragma Import
   --   (C, pthread_attr_setschedpolicy,
   --       "pthread_attr_setsched");
   --  MaRTE OS >>

   function pthread_attr_getschedpolicy
     (attr   : access pthread_attr_t;
      policy : access int)
     return int
     renames Kernel.Tasks_Operations.Attributes.Pthread_Attr_Getschedpolicy;
   --  << MaRTE OS
   --  pragma Import (C, pthread_attr_getschedpolicy,
   --    "pthread_attr_getsched");
   --  MaRTE OS >>

   function sched_yield return int renames Sched.Sched_Yield;
   --  << MaRTE OS
   --  pragma Inline (sched_yield);
   --  MaRTE OS >>
   --  FSU_THREADS does not have sched_yield;

   -----------------------------
   --  P1003.1c - Section 16  --
   -----------------------------

   function pthread_attr_init
     (attributes : access pthread_attr_t)
     return int
     renames Pthread.Pthread_Attr_Init;
   --  << MaRTE OS
   --  pragma Import (C, pthread_attr_init, "pthread_attr_init");
   --  MaRTE OS >>

   function pthread_attr_destroy
     (attributes : access pthread_attr_t)
     return int
     renames Pthread.Pthread_Attr_Destroy;
   --  << MaRTE OS
   --  pragma Import (C, pthread_attr_destroy, "pthread_attr_destroy");
   --  MaRTE OS >>

   function pthread_attr_setdetachstate
     (attr        : access pthread_attr_t;
      detachstate : int)
     return int
     renames Kernel.Tasks_Operations.Attributes.Pthread_Attr_Setdetachstate;
   --  << MaRTE OS
   --  pragma Inline (pthread_attr_setdetachstate);
   --  MaRTE OS >>
   --  FSU_THREADS has a nonstandard pthread_attr_setdetachstate

   function pthread_attr_getdetachstate
     (attr        : access pthread_attr_t;
      detachstate : access int)
     return int
     renames Kernel.Tasks_Operations.Attributes.Pthread_Attr_Getdetachstate;
   --  << MaRTE OS
   --  pragma Import
   --    (C, pthread_attr_getdetachstate, "pthread_attr_getdetachstate");
   --  MaRTE OS >>

   function pthread_attr_getstacksize
     (attr      : access pthread_attr_t;
      stacksize : access size_t)
     return int;
   --  << MaRTE OS
   --  pragma Import
   --    (C, pthread_attr_getstacksize, "pthread_attr_getstacksize");
   --  MaRTE OS >>

   function pthread_attr_setstacksize
     (attr      : access pthread_attr_t;
      stacksize : size_t)
     return int;
   --  << MaRTE OS
   --  pragma Import
   --    (C, pthread_attr_setstacksize, "pthread_attr_setstacksize");
   --  MaRTE OS >>

   function pthread_create
     (thread        : access pthread_t;
      attributes    : access pthread_attr_t;
      start_routine : Thread_Body;
      arg           : System.Address)
     return int;
   --  << MaRTE OS
   --  pragma Import (C, pthread_create, "pthread_create");
   --  MaRTE OS >>

   function pthread_detach (thread : pthread_t) return int;
   --  << MaRTE OS
   --  pragma Inline (pthread_detach);
   --  MaRTE OS >>
   --  FSU_THREADS has a nonstandard pthread_detach

   procedure pthread_exit (status : System.Address)
     renames Pthread.Pthread_Exit;
   --  << MaRTE OS
   --  pragma Import (C, pthread_exit, "pthread_exit");
   --  MaRTE OS >>

   function pthread_self return pthread_t;
   --  << MaRTE OS
   --  pragma Import (C, pthread_self, "pthread_self");
   --  MaRTE OS >>

   function pthread_equal (t1 : pthread_t; t2 : pthread_t)
      return int;
   --  be careful not to use "=" on thread_t!
   --  << MaRTE OS
   --  pragma Import (C, pthread_equal, "pthread_equal");
   --  MaRTE OS >>

   ----------------------------
   --  POSIX.1c  Section 17  --
   ----------------------------

   function pthread_setspecific
     (key   : pthread_key_t;
      value : System.Address)
     return  int
     renames Pthread.Pthread_Setspecific;
   --  << MaRTE OS
   --  pragma Import (C, pthread_setspecific, "pthread_setspecific");
   --  MaRTE OS >>

   function pthread_getspecific (key : pthread_key_t) return System.Address
     renames Pthread.Pthread_Getspecific;
   pragma Inline (pthread_getspecific);
   --  FSU_THREADS has a nonstandard pthread_getspecific

   type destructor_pointer is access
      procedure (arg : System.Address);

   function pthread_key_create
     (key        : access pthread_key_t;
      destructor : destructor_pointer)
     return int;
   --  << MaRTE OS
   --  pragma Import (C, pthread_key_create, "pthread_key_create");
   --  MaRTE OS >>

private

--  << MaRTE OS  --------------------------------------------------------------
--     type sigset_t is array (0 .. 31) of unsigned_long;
--     pragma Convention (C, sigset_t);
--     -- This is for GNU libc version 2 but should be backward compatible with
--     -- other libc where sigset_t is smaller.

--     type pid_t is new int;

--     type time_t is new long;

--     type timespec is record
--        tv_sec       : time_t;
--        tv_nsec      : long;
--     end record;
--     pragma Convention (C, timespec);

--     type clockid_t is new int;
--     CLOCK_REALTIME : constant clockid_t := 0;

--     type struct_timeval is record
--        tv_sec       : long;
--        tv_usec      : long;
--     end record;
--     pragma Convention (C, struct_timeval);

--     type pthread_attr_t is record
--        flags        : int;
--        stacksize    : int;
--        contentionscope  : int;
--        inheritsched  : int;
--        detachstate  : int;
--        sched        : int;
--        prio         : int;
--        starttime    : timespec;
--        deadline     : timespec;
--        period       : timespec;
--     end record;

--     pragma Convention (C_Pass_By_Copy, pthread_attr_t);

--     type pthread_condattr_t is record
--        flags        : int;
--     end record;
--     pragma Convention (C, pthread_condattr_t);

--     type pthread_mutexattr_t is record
--        flags        : int;
--        prio_ceiling  : int;
--        protocol     : int;
--     end record;
--     pragma Convention (C, pthread_mutexattr_t);

--     type sigjmp_buf is array (Integer range 0 .. 38) of int;

--     type array_type_11 is array (Integer range 0 .. 2) of System.Address;
--     type union_type_4 is new int;
--     type record_type_6 is record
--        si_signo     : int;
--        si_code      : int;
--        si_value     : union_type_4;
--     end record;
--     pragma Convention (C, record_type_6);
--     type array_type_12 is array (Integer range 0 .. 44) of record_type_6;
--     type record_type_7 is record
--        head         : System.Address;
--        tail         : System.Address;
--     end record;
--     pragma Convention (C, record_type_7);
--     type array_type_13 is array (Integer range 0 .. 7) of System.Address;
--     type record_type_5 is record
--        context      : sigjmp_buf;
--        pbody        : sigjmp_buf;
--        errno        : int;
--        ret          : int;
--        stack_base   : System.Address;
--        state        : int;
--        next         : array_type_11;
--        num_timers   : int;
--        interval     : struct_timeval;
--        sig_info     : array_type_12;
--        sig          : int;
--        code         : int;
--        osp          : int;
--        opc          : int;
--        obp          : int;
--        nscp         : System.Address;
--        scp          : System.Address;
--        joinq        : record_type_7;
--        cond         : System.Address;
--        queue        : System.Address;
--        mask         : sigset_t;
--        pending      : sigset_t;
--        sigwaitset   : sigset_t;
--        func         : System.Address;
--        arg          : System.Address;
--        result       : System.Address;
--        key          : array_type_13;
--        cleanup_top  : System.Address;
--        attr         : pthread_attr_t;
--        base_prio    : int;
--        max_ceiling_prio  : int;
--        new_prio     : int;
--        tp           : struct_timeval;
--     end record;
--     pragma Convention (C, record_type_5);
--     type pthread_t is access record_type_5;

--     type record_type_3 is record
--        head         : System.Address;
--        tail         : System.Address;
--     end record;
--     pragma Convention (C, record_type_3);
--     type pthread_mutex_t is record
--        queue        : record_type_3;
--        lock         : plain_char;
--        owner        : System.Address;
--        flags        : int;
--        prio_ceiling  : int;
--        protocol     : int;
--        prev_max_ceiling_prio  : int;
--     end record;
--     pragma Convention (C, pthread_mutex_t);

--     type record_type_4 is record
--        head         : System.Address;
--        tail         : System.Address;
--     end record;
--     pragma Convention (C, record_type_4);
--     type pthread_cond_t is record
--        queue        : record_type_4;
--        flags        : int;
--        waiters      : int;
--        mutex        : System.Address;
--     end record;
--     pragma Convention (C, pthread_cond_t);

--     type pthread_key_t is new int;



   type sigset_t  is new PSignal.Signal_Set;


   type pthread_t           is new Pthread.Pthread_T;
   type pthread_mutex_t     is new Pthread.Pthread_Mutex_T;

--  MaRTE OS >>  -------------------------------------------------------------
end System.OS_Interface;
