------------------------------------------------------------------------------
--                                                                          --
--                GNU ADA RUN-TIME LIBRARY (GNARL) COMPONENTS               --
--                                                                          --
--                   S Y S T E M . O S _ I N T E R F A C E                  --
--                                                                          --
--                                  B o d y                                 --
--                                                                          --
--                              $Revision: 53 $
--                                                                          --
--            Copyright (C) 1997-2001 Florida State University              --
--                                                                          --
-- GNARL is free software; you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. GNARL is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with GNARL; see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
-- GNARL was developed by the GNARL team at Florida State University. It is --
-- now maintained by Ada Core Technologies Inc. in cooperation with Florida --
-- State University (http://www.gnat.com).                                  --
--                                                                          --
------------------------------------------------------------------------------

------------------------------------------------------------------------------
--  ------------------         M a R T E   O S         -------------------  --
------------------------------------------------------------------------------
--                                                             V1.61B   070119
--
--  This is a MaRTE OS version of this package. Based in the original
--  GNU/Linux (FSU THREADS) version of this package.
--
--  (Thanks to Javier Miranda (Universidad de Las Palmas de Gran Canaria)
--  for its help in this adaptation).
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2000-2007, Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael Gonzalez Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------


pragma Polling (Off);
--  Turn off polling, we do not want ATC polling to take place during
--  tasking operations. It causes infinite loops and other problems.

with Ada.Unchecked_Conversion;

--  << MaRTE OS
with Hardware_Interface;
with Sched;
pragma Elaborate_All (Sched);
with Time;
pragma Elaborate_All (Time);
with Pthread;
pragma Elaborate_All (Pthread);

with Kernel_Console;
--  MaRTE OS >>

with Interfaces.C;

package body System.OS_Interface is

   use Interfaces.C;
   package HWI renames Hardware_Interface; --  MaRTE OS

   --  << MaRTE OS
--     -----------------
--     -- To_Duration --
--     -----------------

--     function To_Duration (TS : timespec) return Duration is
--     begin
--        return Duration (TS.tv_sec) + Duration (TS.tv_nsec) / 10#1#E9;
--     end To_Duration;

--     function To_Duration (TV : struct_timeval) return Duration is
--     begin
--        return Duration (TV.tv_sec) + Duration (TV.tv_usec) / 10#1#E6;
--     end To_Duration;

--     -----------------
--     -- To_Timespec --
--     -----------------

--     function To_Timespec (D : Duration) return timespec is
--        S : time_t;
--        F : Duration;

--     begin
--        S := time_t (Long_Long_Integer (D));
--        F := D - Duration (S);

--        --  If F has negative value due to a round-up, adjust for positive F
--        --  value.

--        if F < 0.0 then
--           S := S - 1;
--           F := F + 1.0;
--        end if;

--        return timespec' (tv_sec => S,
--          tv_nsec => long (Long_Long_Integer (F * 10#1#E9)));
--     end To_Timespec;

--     ----------------
--     -- To_Timeval --
--     ----------------

--     function To_Timeval (D : Duration) return struct_timeval is
--        S : long;
--        F : Duration;

--     begin
--        S := long (Long_Long_Integer (D));
--        F := D - Duration (S);

--        --  If F has negative value due to a round-up, adjust for positive F
--        --  value.

--        if F < 0.0 then
--           S := S - 1;
--           F := F + 1.0;
--        end if;

--        return struct_timeval' (tv_sec => S,
--          tv_usec => long (Long_Long_Integer (F * 10#1#E6)));
--     end To_Timeval;
   --  MaRTE OS >>

   --  << MaRTE OS
   ----------------------
   -- Type conversions --
   ----------------------
   function To_Descriptor (M : access pthread_mutex_t) return
     Pthread.Mutex_Descriptor;

   function To_Descriptor (M : access pthread_mutex_t) return
     Pthread.Mutex_Descriptor is
   begin
      return Pthread.Descriptor_Of (Pthread.Pthread_Mutex_T (M.all));
   end To_Descriptor;
   pragma Inline (To_Descriptor);

   function To_Descriptor (CV : access pthread_cond_t) return
     Pthread.Condition_Descriptor;

   function To_Descriptor (CV : access pthread_cond_t) return
     Pthread.Condition_Descriptor is
   begin
      return Pthread.Descriptor_Of
        (Pthread.Pthread_Cond_T (CV.all));
   end To_Descriptor;
   pragma Inline (To_Descriptor);

   type Mutex_Ac is access Pthread.Pthread_Mutex_T;
   function To_Access_Mutex (M : access pthread_mutex_t) return Mutex_Ac;
   pragma Import (C, To_Access_Mutex, "convert_4_bytes_long_data");

   function To_Gnat_Pthread_T is
     new Ada.Unchecked_Conversion (Pthread.Pthread_T, pthread_t);
   function To_MaRTE_Pthread_T is
     new Ada.Unchecked_Conversion (pthread_t, Pthread.Pthread_T);

   -----------
   -- Errno --
   -----------
   function errno return int is
   begin
      return Pthread.Pthread_Self.Error_Code;
   end errno;
   --  MaRTE OS >>

   --  << MaRTE OS

   ----------------------------------------------------------------------------
   -- Signals -----------------------------------------------------------------
   ----------------------------------------------------------------------------

   function sigaddset
     (set : access sigset_t;
      sig : in Signal)
      return int is
      Ret : int;
      Set_Tmp : aliased PSignal.Signal_Set := PSignal.Signal_Set (set.all);
   begin
      Ret := PSignal.Sigaddset (Set_Tmp'Access, PSignal.Signal (sig));
      set.all := sigset_t (Set_Tmp);
      return Ret;
   end sigaddset;

   function sigdelset
     (set : access sigset_t;
      sig : in Signal)
      return int is
      Ret : int;
      Set_Tmp : aliased PSignal.Signal_Set := PSignal.Signal_Set (set.all);
   begin
      Ret := PSignal.Sigdelset (Set_Tmp'Access, PSignal.Signal (sig));
      set.all := sigset_t (Set_Tmp);
      return Ret;
   end sigdelset;

   function sigfillset
     (set : access sigset_t)
      return int is
      Ret : int;
      Set_Tmp : aliased PSignal.Signal_Set := PSignal.Signal_Set (set.all);
   begin
      Ret :=  PSignal.Sigfillset (Set_Tmp'Access);
      set.all := sigset_t (Set_Tmp);
      return Ret;
   end sigfillset;

   function sigismember
     (set : access sigset_t;
      sig : Signal)
     return int is
      Ret : int;
      Set_Tmp : aliased PSignal.Signal_Set := PSignal.Signal_Set (set.all);
   begin
      Ret := PSignal.Sigismember (Set_Tmp'Unchecked_Access,
                                  PSignal.Signal (sig));
      set.all := sigset_t (Set_Tmp);
      return Ret;
   end sigismember;

   function sigemptyset
     (set : access sigset_t)
     return int is
      Ret : int;
      Set_Tmp : aliased PSignal.Signal_Set := PSignal.Signal_Set (set.all);
   begin
      Ret :=  PSignal.Sigemptyset (Set_Tmp'Access);
      set.all := sigset_t (Set_Tmp);
      return Ret;
   end sigemptyset;

   ---------------
   -- sigaction --
   ---------------
   function To_Struct_Sig_Action_Ac is
     new Ada.Unchecked_Conversion (struct_sigaction_ptr,
                                   PSignal.Struct_Sig_Action_Ac);
   function To_Struct_Sig_Action_Ac_Const is
     new Ada.Unchecked_Conversion (struct_sigaction_ptr,
                                   PSignal.Struct_Sig_Action_Ac_Const);

   function sigaction (sig  : Signal;
                       act  : struct_sigaction_ptr;
                       oact : struct_sigaction_ptr) return int is
   begin
      return PSignal.Sigaction (PSignal.Signal (sig),
                                To_Struct_Sig_Action_Ac_Const (act),
                                To_Struct_Sig_Action_Ac (oact));
   end sigaction;


   ----------------------------------------------------------------------------
   -- Time --------------------------------------------------------------------
   ----------------------------------------------------------------------------

   function clock_gettime (clock_id : clockid_t;
                           tp       : access timespec) return int is
      use type Time.Clockid_T;
   begin
      pragma Assert (clock_id = CLOCK_REALTIME);
      tp.all := Time.Get_Time_Gnat;
      return 0;
   end clock_gettime;

   function To_Duration (TS : timespec) return Duration is
   begin
      return TS;
   end To_Duration;
   pragma Inline (To_Duration);

   function To_Timespec (D : Duration) return timespec is
   begin
      return D;
   end To_Timespec;
   pragma Inline (To_Timespec);


   ---------------------------------------------------------------------------
   -- Process ----------------------------------------------------------------
   ---------------------------------------------------------------------------

   --  XXX Not Implemented
   function getpid return pid_t is
   begin
      return 0;
   end getpid;

   --  << MaRTE OS
   ----------------------------------------------------------------------------
   -- LWP (MARTE) -------------------------------------------------------------
   ----------------------------------------------------------------------------

   function To_System_Address is
      new Ada.Unchecked_Conversion (pthread_t, System.Address);

   function lwp_self return System.Address is
   --  lwp_self does not exist on this thread library, revert to pthread_self
   --  which is the closest approximation (with getpid).
      Self_ID : pthread_t := pthread_self;
   begin
      return To_System_Address (Self_ID);
   end lwp_self;
   --  MaRTE OS >>


   ----------------------------------------------------------------------------
   -- Stack -------------------------------------------------------------------
   ----------------------------------------------------------------------------


   ----------------
   -- Stack_Base --
   ----------------
--     function Get_Stack_Base (thread : pthread_t) return Address is
--     begin
--        return thread.stack_base;
--     end Get_Stack_Base;
   function Get_Stack_Base (thread : pthread_t) return Address is
   begin
      return System.Null_Address;
      --  returns the stack base of the specified thread.
      --  Only call this function when Stack_Base_Available is True.
   end Get_Stack_Base;
   pragma Inline (Get_Stack_Base);

   function Get_Page_Size return size_t is
   begin
      return 0;
      --  returns the size of a page, or 0 if this is not relevant on this
      --  target
   end Get_Page_Size;

   function Get_Page_Size return Address is
   begin
      return System.Null_Address;
      --  returns the size of a page, or 0 if this is not relevant on this
      --  target
   end Get_Page_Size;

   --  XXX Not Implemented
   function mprotect (addr : Address; len : size_t; prot : int) return int is
   begin
      Kernel_Console.Error ("Called Not Implemented function: Mprotect !!");
      return 0;
   end mprotect;

   ---------------------------------------------------------------------------
   --  Nonstandard Thread Initialization  ------------------------------------
   ---------------------------------------------------------------------------
   --  FSU_THREADS requires pthread_init, which is nonstandard
   --  and this should be invoked during the elaboration of s-taprop.adb
   procedure pthread_init is
   begin
      null;
   end pthread_init;

   ---------------------------------------------------------------------------
   --  POSIX.1c  Section 3  --------------------------------------------------
   ---------------------------------------------------------------------------

   -------------
   -- sigwait --
   -------------

--     --  FSU_THREADS has a nonstandard sigwait
--     function sigwait
--       (set  : access sigset_t;
--        sig  : access Signal) return int is
--        Result : int;
--        function sigwait_base (set : access sigset_t) return int;
--        pragma Import (C, sigwait_base, "sigwait");
--     begin
--        Result := sigwait_base (set);
--        if Result = -1 then
--           sig.all := 0;
--           return errno;
--        end if;
--        sig.all := Signal (Result);
--        return 0;
--     end sigwait;
   ---------------
   --  sigwait  --
   ---------------
   function sigwait
     (set : access sigset_t;
      sig : access Signal)
      return int is
      Sig_Tmp : aliased PSignal.Signal;
      Set_Tmp : aliased PSignal.Signal_Set := PSignal.Signal_Set (set.all);
      Ret : int;
   begin
      Ret := PSignal.Sigwait (Set_Tmp'Unchecked_Access, Sig_Tmp'Access);
      sig.all := Signal (Sig_Tmp);
      return Ret;
   end sigwait;

   --------------------
   --  pthread_kill  --
   --------------------
   function pthread_kill
     (thread : pthread_t;
      sig    : Signal)
      return   int is
   begin
      return PSignal.Pthread_Kill (To_MaRTE_Pthread_T (thread),
                                   PSignal.Signal (sig));
   end pthread_kill;

   -----------------------
   --  pthread_sigmask  --
   -----------------------
   function To_Signal_Set_Ac is
     new Ada.Unchecked_Conversion (sigset_t_ptr, PSignal.Signal_Set_Ac);
   function To_Signal_Set_Ac_Const is
     new Ada.Unchecked_Conversion (sigset_t_ptr, PSignal.Signal_Set_Ac_Const);

   function pthread_sigmask
     (how  : int;
      set  : sigset_t_ptr;
      oset : sigset_t_ptr)
      return int is
   begin
      return PSignal.Pthread_Sigmask (Integer (how),
                                      To_Signal_Set_Ac_Const (set),
                                      To_Signal_Set_Ac (oset));
   end pthread_sigmask;


   ---------------------------------------------------------------------------
   --  POSIX.1c  Section 11  -------------------------------------------------
   ---------------------------------------------------------------------------

   function pthread_mutex_init (mutex : access pthread_mutex_t;
                                attr  : access pthread_mutexattr_t)
                                return int is
   begin
      return Pthread.Pthread_Mutex_Init (To_Access_Mutex (mutex), attr);
   end pthread_mutex_init;

   function pthread_mutex_destroy (mutex : access pthread_mutex_t)
                                   return  int is
   begin
      return Pthread.Pthread_Mutex_Destroy (To_Access_Mutex (mutex));
   end pthread_mutex_destroy;

   ------------------------
   -- pthread_mutex_lock --
   ------------------------
--     --  FSU_THREADS has nonstandard pthread_mutex_lock and unlock.
--     --  It sets errno but the standard Posix requires it to be returned.
--   function pthread_mutex_lock (mutex : access pthread_mutex_t) return int is
--        function pthread_mutex_lock_base
--          (mutex : access pthread_mutex_t) return int;
--        pragma Import (C, pthread_mutex_lock_base, "pthread_mutex_lock");
--        Result : int;
--     begin
--        Result := pthread_mutex_lock_base (mutex);
--        if Result /= 0 then
--           return errno;
--        end if;
--        return 0;
--     end pthread_mutex_lock;
   function pthread_mutex_lock (mutex : access pthread_mutex_t)
                                return int is
   begin
      return Pthread.Pthread_Mutex_Lock (To_Descriptor (mutex));
   end pthread_mutex_lock;

   --------------------------
   -- pthread_mutex_unlock --
   --------------------------
--     function pthread_mutex_unlock
--       (mutex : access pthread_mutex_t) return int   is
--        function pthread_mutex_unlock_base
--          (mutex : access pthread_mutex_t) return int;
--        pragma Import (C, pthread_mutex_unlock_base, "pthread_mutex_unlock");
--        Result : int;
--     begin
--        Result := pthread_mutex_unlock_base (mutex);
--        if Result /= 0 then
--           return errno;
--        end if;
--        return 0;
--     end pthread_mutex_unlock;
   function pthread_mutex_unlock (mutex : access pthread_mutex_t)
                                  return int is
   begin
      return Pthread.Pthread_Mutex_Unlock (To_Descriptor (mutex));
   end pthread_mutex_unlock;


   -----------------------
   -- pthread_cond_wait --
   -----------------------

--     --  FSU_THREADS has a nonstandard pthread_cond_wait.
--     --  The FSU_THREADS version returns EINTR when interrupted.
--     function pthread_cond_wait
--       (cond  : access pthread_cond_t;
--        mutex : access pthread_mutex_t) return int   is
--        function pthread_cond_wait_base
--          (cond  : access pthread_cond_t;
--           mutex : access pthread_mutex_t) return int;
--        pragma Import (C, pthread_cond_wait_base, "pthread_cond_wait");
--        Result : int;
--     begin
--        Result := pthread_cond_wait_base (cond, mutex);
--        if Result = EINTR then
--           return 0;
--        else
--           return Result;
--        end if;
--     end pthread_cond_wait;
   function pthread_cond_wait (cond  : access pthread_cond_t;
                               mutex : access pthread_mutex_t)
                               return  int is
   begin
      return Pthread.Pthread_Cond_Wait (To_Descriptor (cond),
                                        To_Descriptor (mutex));
   end pthread_cond_wait;


   ----------------------------
   -- pthread_cond_timedwait --
   ----------------------------
--     --  FSU_THREADS has a nonstandard pthread_cond_timedwait. The
--     --  FSU_THREADS version returns -1 and set errno to EAGAIN for timeout.
--     function pthread_cond_timedwait
--       (cond    : access pthread_cond_t;
--        mutex   : access pthread_mutex_t;
--        abstime : access timespec) return int
--     is
--        function pthread_cond_timedwait_base
--          (cond    : access pthread_cond_t;
--           mutex   : access pthread_mutex_t;
--           abstime : access timespec) return int;
--    pragma Import (C, pthread_cond_timedwait_base, "pthread_cond_timedwait");
--        Result : int;
--     begin
--        Result := pthread_cond_timedwait_base (cond, mutex, abstime);
--        if Result = -1 then
--           if errno = EAGAIN then
--              return ETIMEDOUT;
--           else
--              return EINVAL;
--           end if;
--        end if;
--        return 0;
--     end pthread_cond_timedwait;
   function pthread_cond_timedwait (cond    : access pthread_cond_t;
                                    mutex   : access pthread_mutex_t;
                                    abstime : access timespec)
                                    return    int is
   begin
      return Pthread.Pthread_Cond_Timedwait_Gnat
        (To_Descriptor (cond),
         To_Descriptor (mutex),
         HWI.Duration_To_HWTime (abstime.all));
   end pthread_cond_timedwait;


   ---------------------------------------------------------------------------
   --  POSIX.1c  Section 13  -------------------------------------------------
   ---------------------------------------------------------------------------

--     ---------------------------
--     -- pthread_setschedparam --
--     ---------------------------

--     --  FSU_THREADS does not have pthread_setschedparam

--     --  This routine returns a non-negative value upon failure
--     --  but the error code can not be set conforming the POSIX standard.

--     function pthread_setschedparam
--       (thread : pthread_t;
--        policy : int;
--        param  : access struct_sched_param) return int
--     is
--        function pthread_setschedattr
--          (thread : pthread_t;
--           attr   : pthread_attr_t) return int;
--        pragma Import (C, pthread_setschedattr, "pthread_setschedattr");

--        attr   : aliased pthread_attr_t;
--        Result : int;

--     begin
--        Result := pthread_attr_init (attr'Access);

--        if Result /= 0 then
--           return Result;
--        end if;

--        attr.sched := policy;

--        --  Short-cut around pthread_attr_setprio

--        attr.prio := param.sched_priority;

--        Result := pthread_setschedattr (thread, attr);

--        if Result /= 0 then
--           return Result;
--        end if;

--        Result := pthread_attr_destroy (attr'Access);

--        if Result /= 0 then
--           return Result;
--        else
--           return 0;
--        end if;
--     end pthread_setschedparam;
   function pthread_setschedparam (thread : pthread_t;
                                   policy : int;
                                   param  : access struct_sched_param)
                                   return int is
   begin
      return Pthread.Pthread_Setschedparam_Gnat
                     (To_MaRTE_Pthread_T (thread),
                      Pthread.Scheduling_Policies (policy),
                      Pthread.Task_Priority (param.sched_priority));
      --  Gnat only knows about the 'sched_priority' param, so the normal
      --  'Pthread.Pthread_Setschedparam' can not be called from here.
   end pthread_setschedparam;

   ----------------------------------------------------------------------------
   --  P1003.1c - Section 16  -------------------------------------------------
   ----------------------------------------------------------------------------

   --  XXX Not Implemented
   function pthread_attr_setstacksize
     (attr     : access pthread_attr_t;
      stacksize : size_t)
      return int is
   begin
      return 0;
   end pthread_attr_setstacksize;

   --------------------
   -- Pthread_Create --
   --------------------
   function To_Task_Body_Function is
     new Ada.Unchecked_Conversion (Thread_Body, Pthread.Task_Body_Function);

   function pthread_create (thread        : access pthread_t;
                            attributes    : access pthread_attr_t;
                            start_routine : Thread_Body;
                            arg           : System.Address)
                            return int is
      Thread_Tmp : aliased Pthread.Pthread_T;
      Ret : int;
   begin
      Ret :=
        Pthread.Pthread_Create (Thread_Tmp'Access,
                                Pthread.Pthread_Attr_T_Ac_Const (attributes),
                                To_Task_Body_Function (start_routine),
                                arg);
      thread.all := To_Gnat_Pthread_T (Thread_Tmp);
      return Ret;
   end pthread_create;

   function pthread_self return pthread_t is
   begin
      return To_Gnat_Pthread_T (Pthread.Pthread_Self);
   end pthread_self;


   ----------------------------------------------------------------------------
   --  POSIX.1c  Section 17  --------------------------------------------------
   ----------------------------------------------------------------------------

   --  Gnat uses the thread specific data functionality to store the gnat
   --  task id (see 'System.Task_Primitives.Operations.Specific.Self').

   Keys_Used_By_Gnat : Integer := 0;
   function Destructor_To_Address is
     new Ada.Unchecked_Conversion (destructor_pointer, System.Address);
   function pthread_key_create
     (key        : access pthread_key_t;
      destructor : destructor_pointer)
      return int is
      Ret  : int;
   begin
      pragma Assert (Keys_Used_By_Gnat = 0,
                     "s-osinte.adb: gnat used more than one key");
      Keys_Used_By_Gnat := Keys_Used_By_Gnat + 1;
      Ret :=
        Pthread.Pthread_Key_Create (key, Destructor_To_Address (destructor));
      Self_Key := key.all;
      --  'Self_Key' is used from
      --  'System.Task_Primitives.Operations.Abort_Handler'.
      return Ret;
   end pthread_key_create;

   --  MaRTE OS >>


   --  << MaRTE OS
--     -------------------------
--     -- pthread_getspecific --
--     -------------------------

--     --  FSU_THREADS has a nonstandard pthread_getspecific

--  function pthread_getspecific (key : pthread_key_t) return System.Address is
--        function pthread_getspecific_base
--          (key   : pthread_key_t;
--           value : access System.Address) return int;
--        pragma Import (C, pthread_getspecific_base, "pthread_getspecific");

--        Tmp    : aliased System.Address;
--        Result : int;

--     begin
--        Result := pthread_getspecific_base (key, Tmp'Access);

--        if Result /= 0 then
--           return System.Null_Address;
--        end if;

--        return Tmp;
--     end pthread_getspecific;

--     ---------------------------------
--     -- pthread_attr_setdetachstate --
--     ---------------------------------

--     function pthread_attr_setdetachstate
--       (attr        : access pthread_attr_t;
--        detachstate : int) return int
--     is
--        function pthread_attr_setdetachstate_base
--          (attr        : access pthread_attr_t;
--           detachstate : access int) return int;
--        pragma Import
--       (C, pthread_attr_setdetachstate_base, "pthread_attr_setdetachstate");

--        Tmp : aliased int := detachstate;

--     begin
--        return pthread_attr_setdetachstate_base (attr, Tmp'Access);
--     end pthread_attr_setdetachstate;

--     -----------------
--     -- sched_yield --
--     -----------------

--     --  FSU_THREADS does not have sched_yield;

--     function sched_yield return int is
--        procedure sched_yield_base (arg : System.Address);
--        pragma Import (C, sched_yield_base, "pthread_yield");

--     begin
--        sched_yield_base (System.Null_Address);
--        return 0;
--     end sched_yield;
   --  MaRTE OS >>

end System.OS_Interface;
