------------------------------------------------------------------------------
--  ------------------        M a R T E     O S        -------------------  --
------------------------------------------------------------------------------
--                                                              V1.61B   070119
--
--                     'I n t e r r u p t _ T a b l e s'
--
--                                 Spec
--
--
--  File 'interrupt_tables.ads'                  By Miguel ngel Masmano Tello
--                                                  and MAR.
--
--
--  Linux interrupt tables management.
--
--  The idea of running MaRTE OS as a Linux process is due to Miguel Angel
--  Masmano     Tello     (Universidad    Politecnica     de     Valencia)
--  <mimastel@doctor.upv.es>.
--  He  is  also  the  author  of   most  of  the  code  involved  in  the
--  implementation of the hardware interface for this architecture.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2000-2007, Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael Gonzalez Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------
with Basic_Integer_Types; use Basic_Integer_Types;

package Interrupt_Tables is

   type Interrupt_Vector is new Integer_32 range 0 .. 255;
   for Interrupt_Vector'Size use 32;

   type Trap_State is record --  PC
      GS : Unsigned_32;
      FS : Unsigned_32;
      ES : Unsigned_32;
      DS : Unsigned_32;

      EDI : Unsigned_32;
      ESI : Unsigned_32;
      EBP : Unsigned_32;
      CR2 : Unsigned_32;
      EBX : Unsigned_32;
      EDX : Unsigned_32;
      ECX : Unsigned_32;
      EAX : Unsigned_32;

      TRAPNO : Unsigned_32;

      ERR : Unsigned_32;

      EIP :    Unsigned_32;
      CS :     Unsigned_32;
      EFlags : Unsigned_32;
      ESP :    Unsigned_32;
      SS :     Unsigned_32;

      V86_ES : Unsigned_32;
      V86_DS : Unsigned_32;
      V86_FS : Unsigned_32;
      V86_GS : Unsigned_32;
   end record;
   pragma Convention (C, Trap_State);
   type Trap_State_Ac is access Trap_State;

   ---------
   -- IRQ --
   ---------
   type IRQ is new Integer_32 range 0 .. 16;
   for IRQ'Size use 32;

   function Linux_Signals_Get_Timer_Signal return IRQ;
   pragma Import (C, Linux_Signals_Get_Timer_Signal,
                    "linux_signals_get_timer_signal");

   TIMER_IRQ       : constant IRQ := Linux_Signals_Get_Timer_Signal;
   --  Signal used for the timer

   type IRQ_Handler_Type is access
     procedure (State : in Trap_State_Ac);

   procedure Inicialize;

end Interrupt_Tables;
