------------------------------------------------------------------------------
--                                   IDE spec                               --
------------------------------------------------------------------------------
--  This module provides a means to read/write sectors to an IDE hard disk. --
--  It doesn't provide concurrency facilities. If you want to use it        --
--  from different tasks you need to provide mutual exclusion access to the --
--  functions. It doesn't make use of DMA facilities, just IO instructions. --
--  How to use it: test_ide.adb                                             --
--                                            daniel.sangorrin_at_gmail.com --
------------------------------------------------------------------------------
with Basic_Integer_Types; use Basic_Integer_Types;
with Kernel.File_System_Data_Types;

package IDE is

   package FSDT renames Kernel.File_System_Data_Types;

   ---------------------
   --  0)  Constants  --
   ---------------------
   SECTOR_SIZE : constant := 512;
   ERROR : constant Int := -1;
   OK : constant Int := 0;

   ----------------------
   --  1)  Data types  --
   ----------------------
   type Drive is (hda, hdb, hdc, hdd);
   type Read_Write is (Read, Write);
   type Sector is array
      (Unsigned_16 range 0 .. IDE.SECTOR_SIZE - 1) of Unsigned_8;
   subtype Sector_Offset is Unsigned_32;
   subtype Sector_Count is Unsigned_8;

   ---------------------
   --  2)  Functions  --
   ---------------------
   function Init (Drv  : in Drive) return Int;
   function Reset (Drv : in Drive) return Int;
   function Free (Drv  : in Drive) return Int;

   function RW_Sectors
     (Drv    : in     Drive;
      RW     : in     Read_Write;
      Start  : in     Sector_Offset; --  physical sector offset
      Count  : in     Sector_Count;  --  0=256 sectors
      Buffer : in     FSDT.Buffer_Ac)
      return Int;

end IDE;