with Basic_Integer_Types;
with Ada.Real_Time;
with Ada.Streams; use Ada.Streams;

with RTEP.Protocol.Servers;
with RTEP_Debug_Functions; use RTEP_Debug_Functions;

package body RTEP.Protocol.C_Interface  is

   procedure Send_Info_C
     (Destination_Station_ID : in Station_ID;
      Channel_ID             : in Channel;
      Data                   : in Buffer_Ac;
      Size                   : in Basic_Integer_Types.Size_T;
      Data_Priority          : in Priority)
   is
      subtype Stream_Data is
         Stream_Element_Array (1 .. Stream_Element_Offset (Size));
      type Stream_Data_Ac is access all Stream_Data;
      function To_Stream_Data_Ac is
         new Ada.Unchecked_Conversion (Buffer_Ac, Stream_Data_Ac);
   begin
      Send_Info (Destination_Station_ID,
                 Channel_ID,
                 To_Stream_Data_Ac (Data).all,
                 Data_Priority);
   end Send_Info_C;

   procedure Send_Info_Server_C
     (Destination_Station_ID : in Station_ID;
      Channel_ID             : in Channel;
      Data                   : in Buffer_Ac;
      Size                   : in Basic_Integer_Types.Size_T;
      Id                     : in Server_ID;
      Blocking               : in Boolean)
   is
      subtype Stream_Data is
         Stream_Element_Array (1 .. Stream_Element_Offset (Size));
      type Stream_Data_Ac is access all Stream_Data;
      function To_Stream_Data_Ac is
         new Ada.Unchecked_Conversion (Buffer_Ac, Stream_Data_Ac);
   begin
      Send_Info
        (Destination_Station_ID,
         Channel_ID,
         To_Stream_Data_Ac (Data).all,
         Id,
         Blocking);
   end Send_Info_Server_C;

   procedure Recv_Info_C
     (Source_Station_ID : out Station_ID;
      Channel_ID        : in Channel;
      Data              : in Buffer_Ac;
      Size              : in Basic_Integer_Types.Size_T;
      Last              : out Basic_Integer_Types.Size_T;
      Data_Priority     : out Priority)
   is
      subtype Stream_Data is
         Stream_Element_Array (1 .. Stream_Element_Offset (Size));
      type Stream_Data_Ac is access all Stream_Data;
      function To_Stream_Data_Ac is
         new Ada.Unchecked_Conversion (Buffer_Ac, Stream_Data_Ac);
      Stream_Last : Stream_Element_Offset;
   begin
      Recv_Info
         (Source_Station_ID,
          Channel_ID,
          To_Stream_Data_Ac (Data).all,
          Stream_Last,
          Data_Priority);
      Last := Basic_Integer_Types.Size_T (Stream_Last);
   end Recv_Info_C;

   function Get_Station_ID_By_Name_C
      (Station_Name : in Buffer_Ac;
       Length       : in Basic_Integer_Types.Size_T) return Station_ID
   is
      subtype String_Station_Name is String (1 .. Integer (Length));
      type String_Station_Name_Ac is access all String_Station_Name;
      function To_String_Station_Name_Ac is
         new Ada.Unchecked_Conversion (Buffer_Ac, String_Station_Name_Ac);
   begin
      return Get_Station_ID_By_Name
         (To_String_Station_Name_Ac (Station_Name).all);
   end Get_Station_ID_By_Name_C;

   procedure Create_Server_C
      (Max_Allocated_Budget : in  Network_Budget;
       Period_Sec           : in  Basic_Integer_Types.Int;
       Period_Nsec          : in  Basic_Integer_Types.Int;
       Server_Priority      : in  RTEP.Priority;
       Id                   : out Server_Id)
   is
      Server_Period : Duration :=
         Duration (Period_Sec) + Duration (Period_Nsec) / 10#1#E9;
   begin
      RTEP.Protocol.Servers.Create_Server
         (Max_Allocated_Budget,
          Ada.Real_Time.To_Time_Span (Server_Period),
          Server_Priority,
          Id);
      DEBUG ("Max_Allocated_Budget: "
         &Network_Budget'Image (Max_Allocated_Budget),
         Enable_DEBUG_C_Interface'First);
      DEBUG ("timespec1: "
         &Basic_Integer_Types.Int'Image (Period_Sec),
         Enable_DEBUG_C_Interface'First);
      DEBUG ("timespec2: "
         &Basic_Integer_Types.Int'Image (Period_Nsec),
         Enable_DEBUG_C_Interface'First);
      DEBUG ("Server_Period: "
         &Duration'Image
            (Duration (Period_Sec) + Duration (Period_Nsec) / 10#1#E9),
            Enable_DEBUG_C_Interface'First);
      DEBUG ("Server_Priority: "&RTEP.Priority'Image (Server_Priority),
         Enable_DEBUG_C_Interface'First);
      DEBUG ("Server: "&Server_ID'Image (Id), Enable_DEBUG_C_Interface'First);
   end Create_Server_C;

   function Init_Distributed_Mutex_C
      (M     : in Distributed_Mutex_Id;
       Group : in Station_Id;
       S_Id  : in Server_ID) return Basic_Integer_Types.Int is
   begin
      RTEP.Protocol.Init_Distributed_Mutex (M, Group, S_Id);
      return 0;
   exception
      when others =>
         return -1;
   end Init_Distributed_Mutex_C;

   function Try_Lock_Distributed_Mutex_C
      (M : Distributed_Mutex_Id) return Basic_Integer_Types.Int is
   begin
      if RTEP.Protocol.Try_Lock_Distributed_Mutex (M) then
         return 0;
      else
         return 1;
      end if;
   exception
      when others =>
         ERROR ("Try lock Distributed Mutex C Interface");
         return -1;
   end Try_Lock_Distributed_Mutex_C;

   function Lock_Distributed_Mutex_C
      (M : Distributed_Mutex_Id) return Basic_Integer_Types.Int is
   begin
      RTEP.Protocol.Lock_Distributed_Mutex (M);
      return 0;
   exception
      when others =>
         ERROR ("Lock Distributed Mutex C Interface");
         return -1;
   end Lock_Distributed_Mutex_C;

   function Unlock_Distributed_Mutex_C
      (M : Distributed_Mutex_Id) return Basic_Integer_Types.Int is
   begin
      RTEP.Protocol.Unlock_Distributed_Mutex (M);
      return 0;
   exception
      when others =>
         ERROR ("Unlock Distributed Mutex C Interface");
         return -1;
   end Unlock_Distributed_Mutex_C;

   function Finalize_Distributed_Mutex_C
      (M : Distributed_Mutex_Id) return Basic_Integer_Types.Int is
   begin
      RTEP.Protocol.Finalize_Distributed_Mutex (M);
      return 0;
   exception
      when others =>
         ERROR ("Finalize Distributed Mutex C Interface");
         return -1;
   end Finalize_Distributed_Mutex_C;

end RTEP.Protocol.C_Interface;
