
#ifndef _RTEP_H_
#define _RTEP_H_

#include <time.h>

extern void rtep_adainit(void);

extern void rtep_adafinal(void);

extern void rtep_init_comm(void);

extern uint16_t rtep_get_station_id(void);

extern void rtep_send_info
   (uint16_t station, uint16_t channel,
    uint8_t *stream, size_t size, uint8_t prio);

extern void rtep_server_send_info
   (uint16_t station, uint16_t channel,
    uint8_t *stream, size_t size, uint16_t server, uint8_t blocking);

extern void rtep_recv_info
   (uint16_t *station, uint16_t channel,
    uint8_t *stream, size_t size, size_t *last, uint8_t *prio);

extern uint16_t rtep_get_station_id_by_name
   (uint8_t *station_name, size_t length);

extern void rtep_server_create
   (uint16_t max_allocated_budget,
    int period_seconds,
    int period_nanoseconds,
    uint8_t prio,
    uint16_t *server);

extern int rtep_init_distributed_mutex
   (uint16_t mutex_id,
    uint16_t multicast_group,
    uint16_t server);

extern int rtep_try_lock_distributed_mutex
   (uint16_t mutex_id);

extern int rtep_lock_distributed_mutex
   (uint16_t mutex_id);

extern int rtep_unlock_distributed_mutex
   (uint16_t mutex_id);

extern int rtep_finalize_distributed_mutex
   (uint16_t mutex_id);

// FNA

extern int rtep_fna_init_ada(void);

typedef struct {
   struct timespec period_max;
   struct timespec deadline;
   uint16_t budget_min;
   uint8_t prio;
} rtep_fna_contract;

int rtep_fna_contract_set_basic_params_ada
   (rtep_fna_contract *contract,
    const uint16_t budget_min,
    const struct timespec *period_max,
    const struct timespec *deadline);

int rtep_fna_set_priority_ada
   (rtep_fna_contract *contract,
    const uint8_t prio);

typedef struct {
   uint16_t contract_id;
   uint16_t server_id;
} rtep_fna_vres;

int rtep_fna_contract_negotiate_ada
   (const rtep_fna_contract *contract,
    rtep_fna_vres *vres);

int rtep_fna_get_server_id_ada
   (const rtep_fna_vres *vres,
    uint16_t *server);

// BEGIN #include <fna.h> and <frsh_*.h>
typedef int frsh_resource_id_t;
typedef rtep_fna_contract frsh_contract_t;
typedef void *fna_vres_id_t;
typedef unsigned int frsh_network_address_t;
typedef unsigned int frsh_stream_id_t;

typedef enum {
   FRSH_QRP_OLDEST,
   FRSH_QRP_NEWCOMER
} frsh_queue_rejection_policy_t;

typedef struct {
   int queue_size;
   frsh_queue_rejection_policy_t queue_policy;
} frsh_endpoint_queueing_info_t;

typedef struct {
   void *body;
   int size;
} frsh_protocol_info_t;

typedef struct {
   frsh_resource_id_t network_id; // int
   frsh_network_address_t destination; // uint32_t
   frsh_stream_id_t stream_id; // int
   fna_vres_id_t vres; // void *
} fna_send_endpoint_t;

typedef struct {
   frsh_resource_id_t network_id; // int
   frsh_stream_id_t stream_id; // int
   frsh_endpoint_queueing_info_t queueing_info;
   frsh_protocol_info_t protocol_info;
} frsh_receive_endpoint_t;

int rtep_fna_init(const frsh_resource_id_t resource_id);

int rtep_fna_contract_negotiate
      (const frsh_resource_id_t resource_id,
       const frsh_contract_t *contract,
       fna_vres_id_t *vres);

int rtep_fna_vres_destroy
      (const frsh_resource_id_t resource_id,
       const fna_vres_id_t vres);

int rtep_fna_vres_destroy_ada
      (const fna_vres_id_t vres);

int rtep_fna_send_async
      (const fna_send_endpoint_t *endpoint,
       const void *msg,
       const size_t size);

int rtep_fna_receive_endpoint_create_callback
      (const frsh_receive_endpoint_t *endpoint);

int rtep_fna_receive_sync
      (const frsh_receive_endpoint_t *endpoint,
       void *buffer,
       const size_t buffer_size,
       size_t *received_bytes);

// END #include <fna.h> and <frsh_*.h>

#endif // _RTEP_H_
