with RTEP; --  for Priority, Server_Id, Number_Of_Servers, Number_Of_Stations
with Ada.Real_Time;
with Type_Timespec; --  for Timespec
pragma Elaborate_All (Type_Timespec);
with Basic_Integer_Types;

package RTEP_FNA is

   package BIT renames Basic_Integer_Types;

   --  Maximum number of accepted contracts for a network (global value)
   Max_Contracts : constant := RTEP.Number_Of_Servers*RTEP.Number_Of_Stations;
   --  Maximum number of pending negotiation requests at a time
   Mx_Neg_Requests : constant := 10;
   --  Channel through where negotiation messages are sent
   Negotiation_Channel : constant RTEP.Channel := 1;
   --  Distributed Mutex used for negotiation
   Negotiation_Mutex   : constant RTEP.Distributed_Mutex_Id := 1;
   --  Set of stations that shares the negotiation mutex
   Negotiation_Mutex_Group  : constant String := "broadcast";
   --  Set of negotiation messages prio
   Negotiation_Msg_Prio  : constant := 18;
   --  Internal Negotiation TASK priority
   Negotiation_Task_Prio : constant := 28;
   --  Internal Negotiation Results TASK priority (higher than the previous one)
   Negotiation_Results_Task_Prio : constant := Negotiation_Task_Prio + 1;
   --  Maximum priority of the tasks involved in negotiations
   Mx_Negotation_Prio : constant := Negotiation_Results_Task_Prio;

   type RTEP_FNA_Contract is private;
   type RTEP_FNA_Virtual_Resource_Id is private;

   function Init return BIT.Int;
   pragma Export (C, Init, "rtep_fna_init_ada");
   pragma Inline (Init);

   function Create_Vres return RTEP_FNA_Virtual_Resource_Id;

   function Set_Basic_Params
      (Contract   : access RTEP_FNA_Contract; --  in out
       Budget_Min : in RTEP.Network_Budget;
       Period_Max : in Type_Timespec.Timespec;
       Deadline   : in Type_Timespec.Timespec) return BIT.Int;
   pragma Export (C, Set_Basic_Params,"rtep_fna_contract_set_basic_params_ada");
   pragma Inline (Set_Basic_Params);

   function Set_Priority
      (Contract : access RTEP_FNA_Contract; --  in out
       Prio     : in RTEP.Priority) return BIT.Int;
   pragma Export (C, Set_Priority,"rtep_fna_set_priority_ada");
   pragma Inline (Set_Priority);

   function Negotiate_Contract
      (Contract   : in  RTEP_FNA_Contract;
       Vres       : in RTEP_FNA_Virtual_Resource_Id)
       return BIT.Int;
   pragma Export (C, Negotiate_Contract,"rtep_fna_contract_negotiate_ada");
   pragma Inline (Negotiate_Contract);

   function Cancel_Contract
     (Vres : in RTEP_FNA_Virtual_Resource_Id) return BIT.Int;
   pragma Export (C, Cancel_Contract, "rtep_fna_vres_destroy_ada");
   pragma Inline (Cancel_Contract);

   function Get_Server_Id
      (Vres : in RTEP_FNA_Virtual_Resource_Id;
       S_Id : access RTEP.Server_Id) --  out
       return BIT.Int;
   pragma Export (C, Get_Server_Id,"rtep_fna_get_server_id_ada");
   pragma Inline (Get_Server_Id);

   function Renegotiate_Contract_Sync
      (Vres     : in RTEP_FNA_Virtual_Resource_Id;
       Contract : in  RTEP_FNA_Contract)
       return BIT.Int;
   pragma Export
      (C, Renegotiate_Contract_Sync, "rtep_fna_contract_renegotiate_sync_ada");
   pragma Inline (Renegotiate_Contract_Sync);

private

   type RTEP_FNA_Contract is record
      Period_Max : Type_Timespec.Timespec;
      Deadline   : Type_Timespec.Timespec;
      Budget_Min : RTEP.Network_Budget;
      Priority   : RTEP.Priority;
   end record;
   pragma Pack (RTEP_FNA_Contract);
   RTEP_FNA_Contract_Size : constant := 2 + 8 + 8 + 1;
   for RTEP_FNA_Contract'Size use RTEP_FNA_Contract_Size * 8;

   type RTEP_FNA_Contract_Id is range 1 .. Max_Contracts;
   for RTEP_FNA_Contract_Id'Size use 16;

   type RTEP_FNA_Virtual_Resource is record
      Contract_Id : RTEP_FNA_Contract_Id;
      Server      : RTEP.Server_Id;
   end record;
   pragma Convention (C, RTEP_FNA_Virtual_Resource);
   pragma Pack (RTEP_FNA_Virtual_Resource);
   for RTEP_FNA_Virtual_Resource'Size use (2 + 2)*8;

   type RTEP_FNA_Virtual_Resource_Id is access RTEP_FNA_Virtual_Resource;

end RTEP_FNA;