#include <stdio.h>
#include <stdint.h>
#include <malloc.h>
#include <misc/error_checks.h>
#include "../rtep.h"

int rtep_fna_init(const frsh_resource_id_t resource_id)
{
   /* RTEP is not capable of handling several network instances yet */
   return rtep_fna_init_ada();
}

int rtep_fna_contract_negotiate
   (const frsh_resource_id_t resource_id,
    const frsh_contract_t *contract,
    fna_vres_id_t *vres)
{
   rtep_fna_contract tmp_contract;
   rtep_fna_vres *tmp_vres;
   int accepted;

   /* convert the FRSH contract to our internal contract type */
   tmp_contract.period_max = contract->period_max;
   tmp_contract.deadline = contract->deadline;
   tmp_contract.budget_min = contract->budget_min;
   tmp_contract.prio = contract->prio;

   /* create an internal vres. The fna vres id is a pointer to void which
   we transform internally to a pointer to a vres structure */
   tmp_vres = (rtep_fna_vres *)malloc(sizeof(rtep_fna_vres));

   /* negotiate the contract through the Ada function */
   accepted = rtep_fna_contract_negotiate_ada (&tmp_contract, tmp_vres);

   if (accepted == 0){
      *vres = (void *)tmp_vres;
   }else{
      free(tmp_vres);
   }

   return accepted;
}

int rtep_fna_vres_destroy
      (const frsh_resource_id_t resource_id,
       const fna_vres_id_t vres)
{
   /* RTEP is not capable of handling several network instances yet */
   if (rtep_fna_vres_destroy_ada(vres) != 0){
      return -1;
   }
   free(vres);
   return 0;
}

int rtep_fna_send_async
   (const fna_send_endpoint_t *endpoint,
    const void *msg,
    const size_t size)
{
   uint16_t server;
   int ret;

   ret = rtep_fna_get_server_id_ada
      ((rtep_fna_vres *)endpoint->vres, &server);

   if (ret != 0)
      return ret;

   rtep_server_send_info (
      (uint16_t)endpoint->destination,
      (uint16_t)endpoint->stream_id,
      (uint8_t *)msg,
      size,
      server,
      0);
   return 0;
}

int rtep_fna_receive_endpoint_create_callback
      (const frsh_receive_endpoint_t *endpoint)
{
   return 0;
}

int rtep_fna_receive_sync
      (const frsh_receive_endpoint_t *endpoint,
       void *buffer,
       const size_t buffer_size,
       size_t *received_bytes)
{
   uint16_t station;
   uint8_t prio;

   rtep_recv_info
      (&station, (uint16_t)endpoint->stream_id, (uint8_t *)buffer, buffer_size,
      received_bytes, &prio);

   return 0;
}
