#include <pthread.h>
#include <stdio.h>
#include <stdint.h>
#include <misc/error_checks.h>
#include <unistd.h>
#include "../rtep.h"

/* receiver_task */
void *receiver_task (void *arg)
{
   uint16_t station;
   uint8_t buffer[20];
   size_t last;
   uint8_t prio;

   printf ("receiver_task\n");
   while (1) {
      rtep_recv_info (&station, 3, buffer, sizeof(buffer), &last, &prio);
      printf ("received data from: %u  with prio: %u last: %d data: ",
              station, prio, last);
      printf ("%s\n", buffer);
   }
}

int main ()
{
   uint16_t me;
   uint16_t dest_station;
   char dest_station_name[] = "broadcast";
   char msg_broadcast[] = "Hi All!";
   rtep_fna_contract contract;
   uint16_t budget_min = 5;
   struct timespec period_max = {10, 0};
   struct timespec deadline = period_max;
   rtep_fna_vres vres;
   uint16_t server;
   uint8_t prio = 8;
   uint16_t channel = 3;
   int accepted;
   pthread_t th;
   pthread_attr_t attr;

   /* initialization */
   rtep_adainit();
   CHKE_FATAL (rtep_fna_init_ada());

   me = rtep_get_station_id();
   printf ("I am: %u\n", me);

   CHKE_FATAL( pthread_attr_init (&attr) );
   CHKE_FATAL( pthread_create (&th, &attr, receiver_task, NULL) );

   contract.budget_min = 69;

   CHKE_FATAL (rtep_fna_contract_set_basic_params_ada
      (&contract, budget_min, &period_max, &deadline));

   CHKE_FATAL (rtep_fna_set_priority_ada (&contract, prio));

   accepted = rtep_fna_contract_negotiate_ada (&contract, &vres);

   if (accepted != 0) {
      printf ("contract not accepted\n");
      return -1;
   }
   printf ("contract accepted\n");

   printf ("server_id: %u\n", vres.server_id);

   CHKE_FATAL (rtep_fna_get_server_id_ada (&vres, &server));

   printf ("server: %u\n", server);

   dest_station = rtep_get_station_id_by_name
      ((uint8_t *)dest_station_name, sizeof(dest_station_name)-1);

   while (1) {
      printf ("sending a message\n");
      rtep_server_send_info
         (dest_station, channel, (uint8_t *)msg_broadcast,
         sizeof(msg_broadcast), server, 0);
      sleep (2);
   }

   // finalization
   rtep_adafinal();
   return 0;
}
