------------------------------------------------------------------------------
--  ------------------        M a R T E     O S        -------------------  --
------------------------------------------------------------------------------
--                                                             V1.61B   070119
--
--                     'I n t e r r u p t _ T a b l e s'
--
--                                 Spec
--
--
--  File 'interrupt_tables.ads'                                        By MAR.
--
--
--  PC interrupt tables management.
--
--  ----------------------------------------------------------------------
--   Copyright (C) 2000-2007, Universidad de Cantabria, SPAIN
--
--   MaRTE OS web page: http://marte.unican.es
--   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
--                      Michael Gonzalez Harbour      mgh@unican.es
--
--  MaRTE OS  is free software; you can  redistribute it and/or  modify it
--  under the terms of the GNU General Public License  as published by the
--  Free Software Foundation;  either  version 2, or (at  your option) any
--  later version.
--
--  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
--  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
--  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
--  General Public License for more details.
--
--  You should have received  a  copy of  the  GNU General Public  License
--  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
--  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
--  02111-1307, USA.
--
--  As a  special exception, if you  link this  unit  with other  files to
--  produce an   executable,   this unit  does  not  by  itself cause  the
--  resulting executable to be covered by the  GNU General Public License.
--  This exception does  not however invalidate  any other reasons why the
--  executable file might be covered by the GNU Public License.
--
------------------------------------------------------------------------------
with Basic_Integer_Types; use Basic_Integer_Types;

package Interrupt_Tables is

   type Interrupt_Vector is new Integer_32 range 0 .. 255;
   for Interrupt_Vector'Size use 32;

   type Trap_State is record --  PC
      GS : Unsigned_32;
      FS : Unsigned_32;
      ES : Unsigned_32;
      DS : Unsigned_32;

      EDI : Unsigned_32;
      ESI : Unsigned_32;
      EBP : Unsigned_32;
      CR2 : Unsigned_32;
      EBX : Unsigned_32;
      EDX : Unsigned_32;
      ECX : Unsigned_32;
      EAX : Unsigned_32;

      TRAPNO : Unsigned_32;

      ERR : Unsigned_32;

      EIP :    Unsigned_32;
      CS :     Unsigned_32;
      EFlags : Unsigned_32;
      ESP :    Unsigned_32;
      SS :     Unsigned_32;

      V86_ES : Unsigned_32;
      V86_DS : Unsigned_32;
      V86_FS : Unsigned_32;
      V86_GS : Unsigned_32;
   end record;
   pragma Convention (C, Trap_State);
   type Trap_State_Ac is access Trap_State;

   type Trap_Handler_Type is access
     function (State : in Trap_State_Ac) return Integer;
   procedure Install_Trap_Handler
     (Vector  : in Interrupt_Vector;
      Handler : in Trap_Handler_Type);
   pragma Import (C, Install_Trap_Handler, "install_trap_handler");
   --  defined in 'interrupt_tables.c'
   ---------
   -- IRQ --
   ---------
   type IRQ is new Integer_32 range 0 .. 16;
   for IRQ'Size use 32;
   --  Hardware interrupts: PCs have 2 PICs with 8 IRQ each plus the local
   --  APIC timer interrupt.

   TIMER_IRQ       : constant IRQ := 0; --  Master PIC
   KEYBOARD_IRQ    : constant IRQ := 1; --  Master PIC
   CTLR2_IRQ       : constant IRQ := 2; --  Master PIC
   SERIAL2_IRQ     : constant IRQ := 3; --  Master PIC
   SERIAL1_IRQ     : constant IRQ := 4; --  Master PIC
   PARALLEL2_IRQ   : constant IRQ := 5; --  Master PIC
   DISKETTE_IRQ    : constant IRQ := 6; --  Master PIC
   PARALLEL1_IRQ   : constant IRQ := 7; --  Master PIC
   RTC_IRQ         : constant IRQ := 8;  --  Slave PIC
   SOFT_IRQ        : constant IRQ := 9;  --  Slave PIC
   RESERVED1_IRQ   : constant IRQ := 10; --  Slave PIC
   RESERVED2_IRQ   : constant IRQ := 11; --  Slave PIC
   RESERVED3_IRQ   : constant IRQ := 12; --  Slave PIC
   COPROCESSOR_IRQ : constant IRQ := 13; --  Slave PIC
   FIXED_DISK_IRQ  : constant IRQ := 14; --  Slave PIC
   RESERVED4_IRQ   : constant IRQ := 15; --  Slave PIC
   LOCAL_APIC_TIMER_IRQ : constant IRQ := 16; -- Local APIC timer
   --  If it is changed the value of 'LOCAL_APIC_TIMER_IRQ' changes
   --  should be performed in 'x86/boot/base_irq_inittab.S' as well.

   type IRQ_Handler_Type is access
     procedure (State : in Trap_State_Ac);
   -------------------------
   -- Install_IRQ_Handler --
   -------------------------
   procedure Install_IRQ_Handler
     (Interrupt : in IRQ;
      Handler   : in IRQ_Handler_Type);
   pragma Import (C, Install_IRQ_Handler, "install_irq_handler");
   --  defined in 'interrupt_tables.c'
   procedure Inicialize (Base_IDT : in Unsigned_16); --  XXX Non implemented
end Interrupt_Tables;
